@@ -25,11 +25,11 @@
 #include <mtm-config-gui.h>
 #include <gtk/gtksignal.h>
 
-static GtkObjectClass *mtm_config_gui_parent_class;
+static GObjectClass *mtm_config_gui_parent_class;
 
 struct _MtmConfigGuiPrivate
 {
-	gint ext_changed;
+	gulong ext_changed;
 };
 
 enum
@@ -51,10 +51,8 @@ enum
 
 guint mtm_config_gui_signals[LAST_SIGNAL] = {0 };
 
-static void mtm_config_gui_set_arg (GtkObject *object, GtkArg *arg,
-				    guint arg_id);
-static void mtm_config_gui_get_arg (GtkObject *object, GtkArg *arg,
-				    guint arg_id);
+static void mtm_config_gui_set_arg (GObject *object, guint arg_id, const GValue *value, GParamSpec *spec);
+static void mtm_config_gui_get_arg (GObject *object, guint arg_id, GValue *value, GParamSpec *spec);
 
 static gchar* mtm_config_gui_browse (MtmConfigGui *gui);
 static void mtm_config_gui_ext_changed (MtmConfigGui *gui, MtmExt *ext);
@@ -63,78 +61,110 @@ static gchar** mtm_strdupv (const gchar** list);
 
 /* Destroy handler for MtmConfigGui */
 static void
-mtm_config_gui_destroy (GtkObject *object)
+mtm_config_gui_dispose (GObject *object)
 {
 	MtmConfigGui *gui = MTM_CONFIG_GUI (object);
 	
 	if (gui->file_list)
+	{
 		g_strfreev (gui->file_list);
+		gui->file_list = NULL;
+	}
 	
 	if (gui->ext)
 	{
-		if (gui->priv->ext_changed != -1)
-			gtk_signal_disconnect (GTK_OBJECT (gui->ext),
-					       gui->priv->ext_changed);
-		gtk_object_unref (GTK_OBJECT (gui->ext));
+		if (gui->priv->ext_changed)
+			g_signal_handler_disconnect (G_OBJECT (gui->ext),
+					     gui->priv->ext_changed);
+		g_object_unref (G_OBJECT (gui->ext));
+		gui->ext = NULL;
 	}
 
-	g_free (gui->priv);
+	mtm_config_gui_parent_class->dispose (object);
+}
 
-	if (GTK_OBJECT_CLASS (mtm_config_gui_parent_class)->destroy)
-		(*GTK_OBJECT_CLASS (mtm_config_gui_parent_class)->destroy)(object);
+static void
+mtm_config_gui_finalize (GObject *object)
+{
+	MtmConfigGui *gui = MTM_CONFIG_GUI (object);
+	
+	g_free (gui->priv);
+	
+	mtm_config_gui_parent_class->finalize (object);
 }
 
 /* Class initialization function for MtmConfigGui */
 static void
-mtm_config_gui_class_init (GtkObjectClass *object_class)
+mtm_config_gui_class_init (GObjectClass *object_class)
 {
 	mtm_config_gui_parent_class = gtk_type_class (gtk_object_get_type ());
 
 	MTM_CONFIG_GUI_CLASS (object_class)->browse = mtm_config_gui_browse;
+	object_class->set_property = mtm_config_gui_set_arg;
+	object_class->get_property = mtm_config_gui_get_arg;
+	object_class->dispose = mtm_config_gui_dispose;
+	object_class->finalize = mtm_config_gui_finalize;
 
 	mtm_config_gui_signals[SET_EXT] =
-		gtk_signal_new ("set_ext",
-			GTK_RUN_LAST, G_OBJECT_CLASS_TYPE (object_class),
-			GTK_SIGNAL_OFFSET (MtmConfigGuiClass, set_ext),
-			gtk_marshal_NONE__OBJECT, GTK_TYPE_NONE, 1,
-			MTM_EXT_TYPE);
+		g_signal_new ("set_ext",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (MtmConfigGuiClass, set_ext),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, MTM_EXT_TYPE);
 
 	mtm_config_gui_signals[SET_CONFIG_AREA] =
-		gtk_signal_new ("set_config_area",
-			GTK_RUN_LAST, G_OBJECT_CLASS_TYPE (object_class),
-			GTK_SIGNAL_OFFSET (MtmConfigGuiClass, set_config_area),
-			gtk_marshal_NONE__OBJECT, GTK_TYPE_NONE, 1,
-			GTK_TYPE_WIDGET);
+		g_signal_new ("set_config_area",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (MtmConfigGuiClass, set_config_area),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
 
 	mtm_config_gui_signals[SET_FILE_LIST] =
-		gtk_signal_new ("set_file_list",
-			GTK_RUN_LAST, G_OBJECT_CLASS_TYPE (object_class),
-			GTK_SIGNAL_OFFSET (MtmConfigGuiClass, set_file_list),
-			gtk_marshal_NONE__OBJECT, GTK_TYPE_NONE, 1,
-			GTK_TYPE_POINTER);
+		g_signal_new ("set_file_list",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (MtmConfigGuiClass, set_file_list),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	mtm_config_gui_signals[EXT_MODIFIED] =
-		gtk_signal_new ("ext_modified",
-			GTK_RUN_LAST, G_OBJECT_CLASS_TYPE (object_class),
-			GTK_SIGNAL_OFFSET (MtmConfigGuiClass, ext_modified),
-			gtk_marshal_NONE__OBJECT, GTK_TYPE_NONE, 1,
-			MTM_EXT_TYPE);
+		g_signal_new ("ext_modified",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (MtmConfigGuiClass, ext_modified),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, MTM_EXT_TYPE);
 
-	object_class->set_arg = mtm_config_gui_set_arg;
-	object_class->get_arg = mtm_config_gui_get_arg;
-
-	gtk_object_add_arg_type ("MtmConfigGui::ext", MTM_EXT_TYPE,
-		GTK_ARG_READWRITE, ARG_EXT);
-	gtk_object_add_arg_type ("MtmConfigGui::file_list", GTK_TYPE_POINTER,
-		GTK_ARG_READWRITE, ARG_FILE_LIST);
-	gtk_object_add_arg_type ("MtmConfigGui::config_area", GTK_TYPE_WIDGET,
-		GTK_ARG_READWRITE, ARG_CONFIG_AREA);
-	
-	object_class->destroy = mtm_config_gui_destroy;
+	g_object_class_install_property
+		(object_class, ARG_EXT,
+		 g_param_spec_object ("ext",
+			 	      "ext",
+				      "ext",
+				      MTM_EXT_TYPE,
+				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_FILE_LIST,
+		 g_param_spec_pointer ("file_list",
+			 	       "file_list",
+				       "file_list",
+				      G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_CONFIG_AREA,
+		 g_param_spec_object ("config_area",
+			 	      "config_area",
+				      "config_area",
+				      GTK_TYPE_WIDGET,
+				      G_PARAM_READWRITE));
 }
 
 static void
-mtm_config_gui_init (GtkObject *object)
+mtm_config_gui_init (GObject *object)
 {
 	MtmConfigGui *gui = MTM_CONFIG_GUI (object);
 	
@@ -142,7 +172,7 @@ mtm_config_gui_init (GtkObject *object)
 
 	gui->config_area = NULL;	
 	gui->file_list = NULL;
-	gui->priv->ext_changed = -1;
+	gui->priv->ext_changed = 0;
 }
 
 
@@ -155,26 +185,30 @@ mtm_config_gui_init (GtkObject *object)
  *
  * Return value: The type ID of the #MtmConfigGui class.
  */
-GtkType
+GType
 mtm_config_gui_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 
 	if (!type)
 	{
-		GtkTypeInfo info =
+		GTypeInfo info =
 		{
-			"MtmConfigGui",
-			sizeof (MtmConfigGui),
 			sizeof (MtmConfigGuiClass),
-			(GtkClassInitFunc) mtm_config_gui_class_init,
-			(GtkObjectInitFunc) mtm_config_gui_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL
+			NULL,
+			NULL,
+			(GClassInitFunc) mtm_config_gui_class_init,
+			NULL,
+			NULL,
+			sizeof (MtmConfigGui),
+			0,	
+			(GInstanceInitFunc) mtm_config_gui_init,
+			NULL
 		};
 
-		type = gtk_type_unique (gtk_object_get_type (), &info);
+		type = g_type_register_static (GTK_TYPE_OBJECT,
+					       "MtmConfigGui",
+					       &info, 0);
 	}
 
 	return type;
@@ -193,7 +227,7 @@ mtm_config_gui_new (void)
 {
 	MtmConfigGui *gui;
 
-	gui = gtk_type_new (mtm_config_gui_get_type ());
+	gui = g_object_new (mtm_config_gui_get_type (), NULL);
 	return gui;
 }
 
@@ -266,25 +300,24 @@ mtm_config_gui_set_ext (MtmConfigGui *gui, MtmExt *ext)
 	
 	/* We disconnect before emitting so as to prevent confusion in
 	 * set_ext signal handlers */
-	if (gui->ext && gui->priv->ext_changed != -1)
-			gtk_signal_disconnect (GTK_OBJECT (gui->ext),
+	if (gui->ext && gui->priv->ext_changed)
+			g_signal_handler_disconnect (G_OBJECT (gui->ext),
 					       gui->priv->ext_changed);
 
 	/* We emit first in case any objects need to know the old object */
-	gtk_signal_emit (GTK_OBJECT (gui),
-		mtm_config_gui_signals[SET_EXT],
-		ext);
+	g_signal_emit (G_OBJECT (gui), mtm_config_gui_signals[SET_EXT],
+		       0, ext);
 
 	if (gui->ext)
-		gtk_object_unref (GTK_OBJECT (gui->ext));
+		g_object_unref (G_OBJECT (gui->ext));
 
 	gui->ext = ext;
-	gtk_object_ref (GTK_OBJECT (ext));
+	g_object_ref (G_OBJECT (ext));
 	gtk_object_sink (GTK_OBJECT (ext));
-	gui->priv->ext_changed = gtk_signal_connect_object (
-					GTK_OBJECT (gui->ext),
+	gui->priv->ext_changed = g_signal_connect_swapped (
+					G_OBJECT (gui->ext),
 					"changed",
-					GTK_SIGNAL_FUNC (mtm_config_gui_ext_changed),
+					(GCallback) mtm_config_gui_ext_changed,
 					GTK_OBJECT (gui));
 }
 
@@ -297,38 +330,37 @@ mtm_config_gui_get_ext (MtmConfigGui *gui)
 }
 
 static void
-mtm_config_gui_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+mtm_config_gui_set_arg (GObject *object, guint arg_id, const GValue *value, GParamSpec *spec)
 {
 	MtmConfigGui *gui = MTM_CONFIG_GUI (object);
 
 	switch (arg_id)
 	{
 		case ARG_EXT:
-			mtm_config_gui_set_ext (gui, MTM_EXT (GTK_VALUE_OBJECT (*arg)));
+			mtm_config_gui_set_ext (gui, MTM_EXT (g_value_get_object (value)));
 			break;
 		case ARG_FILE_LIST:
-			mtm_config_gui_set_file_list (gui, GTK_VALUE_POINTER (*arg));
+			mtm_config_gui_set_file_list (gui, g_value_get_pointer (value));
 			break;
 		case ARG_CONFIG_AREA:
-			mtm_config_gui_set_config_area (gui, GTK_WIDGET (GTK_VALUE_OBJECT (*arg)));
+			mtm_config_gui_set_config_area (gui, GTK_WIDGET (g_value_get_object (value)));
 	}
 }
 
-static void
-mtm_config_gui_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+static void mtm_config_gui_get_arg (GObject *object, guint arg_id, GValue *value, GParamSpec *spec)
 {
 	MtmConfigGui *gui = MTM_CONFIG_GUI (object);
 
 	switch (arg_id)
 	{
 		case ARG_EXT:
-			GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (gui->ext);
+			g_value_set_object (value, G_OBJECT (gui->ext));
 			break;
 		case ARG_FILE_LIST:
-			GTK_VALUE_POINTER (*arg) = gui->file_list;
+			g_value_set_pointer (value, gui->file_list);
 			break;
 		case ARG_CONFIG_AREA:
-			GTK_VALUE_POINTER (*arg) = GTK_OBJECT (gui->config_area);
+			g_value_set_object (value, G_OBJECT (gui->config_area));
 			break;
 	}
 }
@@ -338,9 +370,8 @@ mtm_config_gui_ext_changed (MtmConfigGui *gui, MtmExt *ext)
 {
 	g_return_if_fail (MTM_IS_CONFIG_GUI (gui));
 	
-	gtk_signal_emit (GTK_OBJECT (gui),
-		mtm_config_gui_signals[EXT_MODIFIED],
-		ext);
+	g_signal_emit (GTK_OBJECT (gui), mtm_config_gui_signals[EXT_MODIFIED],
+		       0, ext);
 }
 
 static gchar* 
@@ -365,12 +396,14 @@ mtm_config_gui_set_config_area (MtmConfigGui *gui, GtkWidget *config_area)
 	g_return_if_fail (MTM_IS_CONFIG_GUI (gui));
 	g_return_if_fail (GTK_IS_WIDGET (config_area));
 
-	gtk_signal_emit (GTK_OBJECT (gui), mtm_config_gui_signals[SET_CONFIG_AREA], config_area);
+	g_signal_emit (G_OBJECT (gui),
+		       mtm_config_gui_signals[SET_CONFIG_AREA],
+		       0, config_area);
 	if (gui->config_area)
-		gtk_object_unref (GTK_OBJECT (gui->config_area));
+		g_object_unref (G_OBJECT (gui->config_area));
 
 	gui->config_area = config_area;
-	gtk_object_ref (GTK_OBJECT (config_area));
+	g_object_ref (G_OBJECT (config_area));
 	gtk_object_sink (GTK_OBJECT (config_area));
 }
 
