@@ -37,7 +37,7 @@ struct _MtmPluginClass
 	MtmStatefulClass parent_class;
 };
 
-GtkType    mtm_plugin_get_type (void);
+GType    mtm_plugin_get_type (void);
 MtmPlugin* mtm_plugin_new (MtmEnv *env, const gchar *filename);
 void       mtm_plugin_construct (MtmPlugin *plugin, MtmEnv *env, const gchar *filename);
 
