@@ -25,8 +25,9 @@
 #include <libgnome/libgnome.h>
 #include "bg-ext-handler.h"
 #include <gdk/gdkcolor.h>
+#include <string.h>
 
-static MtmExtHandlerClass *bg_ext_handler_parent_class;
+static GObjectClass *bg_ext_handler_parent_class;
 
 struct _BgExtHandlerPrivate
 {
@@ -95,13 +96,11 @@ bg_grad_get_type (void)
 	return type;
 }
 
-static void bg_ext_handler_set_arg (GtkObject *object, GtkArg *arg,
-	guint arg_id);
-static void bg_ext_handler_get_arg (GtkObject *object, GtkArg *arg,
-	guint arg_id);
+static void bg_ext_handler_set_arg (GObject *object, guint arg_id, const GValue *value, GParamSpec *spec);
+static void bg_ext_handler_get_arg (GObject *object, guint arg_id, GValue *value, GParamSpec *spec);
 
 static void
-bg_ext_handler_destroy (GtkObject *object)
+bg_ext_handler_dispose (GObject *object)
 {
 	BgExtHandler *handler = BG_EXT_HANDLER (object);
 
@@ -109,40 +108,89 @@ bg_ext_handler_destroy (GtkObject *object)
 	{
 		g_list_foreach (handler->priv->configs, (GFunc) g_object_unref, NULL);
 		g_list_free (handler->priv->configs);
+		handler->priv->configs = NULL;
 	}
 	
-	g_hash_table_destroy (handler->priv->descs);
-	g_list_foreach (handler->priv->strs, (GFunc) g_free, NULL);
-	g_list_free (handler->priv->strs);
+	if (handler->priv->descs)
+	{
+		g_hash_table_destroy (handler->priv->descs);
+		handler->priv->descs = NULL;
+	}
 
-	g_free (handler->priv);
+	if (handler->priv->strs)
+	{
+		g_list_foreach (handler->priv->strs, (GFunc) g_free, NULL);
+		g_list_free (handler->priv->strs);
+		handler->priv->strs = NULL;
+	}
 
-	if (GTK_OBJECT_CLASS (bg_ext_handler_parent_class)->destroy)
-		(*GTK_OBJECT_CLASS (bg_ext_handler_parent_class)->destroy)(object);
+	bg_ext_handler_parent_class->dispose (object);
 }
 
 static void
-bg_ext_handler_class_init (GtkObjectClass *object_class)
+bg_ext_handler_finalize (GObject *object)
+{
+	BgExtHandler *handler = BG_EXT_HANDLER (object);
+
+	g_free (handler->priv);
+
+	bg_ext_handler_parent_class->finalize (object);
+}
+
+static void
+bg_ext_handler_class_init (GObjectClass *object_class)
 {
 	bg_ext_handler_parent_class = gtk_type_class (mtm_ext_handler_get_type ());
 
-	object_class->set_arg = bg_ext_handler_set_arg;
-	object_class->get_arg = bg_ext_handler_get_arg;
+	object_class->set_property = bg_ext_handler_set_arg;
+	object_class->get_property= bg_ext_handler_get_arg;
+	object_class->dispose = bg_ext_handler_dispose;
+	object_class->finalize = bg_ext_handler_finalize;
 
-	gtk_object_add_arg_type ("BgExtHandler::Color1", gdk_color_get_type (),
-		GTK_ARG_READWRITE, ARG_COLOR1);
-	gtk_object_add_arg_type ("BgExtHandler::Color2", gdk_color_get_type (),
-		GTK_ARG_READWRITE, ARG_COLOR2);
-	gtk_object_add_arg_type ("BgExtHandler::ColorGradient", GTK_TYPE_BOOL,
-		GTK_ARG_READWRITE, ARG_COLOR_GRADIENT);
-	gtk_object_add_arg_type ("BgExtHandler::GradientType",
-		bg_grad_get_type (), GTK_ARG_READWRITE, ARG_GRADIENT_TYPE);
-	gtk_object_add_arg_type ("BgExtHandler::WallpaperLayout",
-		bg_wp_get_type (), GTK_ARG_READWRITE, ARG_WALLPAPER_LAYOUT);
-	gtk_object_add_arg_type ("BgExtHandler::ext_context",
-		MTM_EXT_TYPE, GTK_ARG_READWRITE, ARG_EXT_CONTEXT);
-
-	object_class->destroy = bg_ext_handler_destroy;
+	g_object_class_install_property
+		(object_class, ARG_COLOR1,
+		 g_param_spec_boxed ("Color1",
+			 	     "Color1",
+				     "Color1",
+				     gdk_color_get_type (),
+				     G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_COLOR2,
+		 g_param_spec_boxed ("Color2",
+			 	     "Color2",
+				     "Color2",
+				     gdk_color_get_type (),
+				     G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_COLOR_GRADIENT,
+		 g_param_spec_boolean ("ColorGradient",
+			 	       "ColorGradient",
+				       "ColorGradient",
+				       FALSE,
+				       G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_GRADIENT_TYPE,
+		 g_param_spec_enum ("GradientType",
+			 	    "GradientType",
+				    "GradientType",
+				    bg_grad_get_type (),
+				    0,
+				    G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_WALLPAPER_LAYOUT,
+		 g_param_spec_enum ("WallpaperLayout",
+			 	    "WallpaperLayout",
+				    "WallpaperLayout",
+				    bg_wp_get_type (),
+				    0,
+				    G_PARAM_READWRITE));
+	g_object_class_install_property
+		(object_class, ARG_EXT_CONTEXT,
+		 g_param_spec_object ("ext_context",
+			 	      "ext_context",
+				      "ext_context",
+				      MTM_EXT_TYPE,
+				      G_PARAM_READWRITE));
 }
 
 static void
@@ -175,27 +223,31 @@ bg_ext_handler_init (GtkObject *object)
 	mtm_ext_handler_register_args (MTM_EXT_HANDLER (handler), args);
 }
 
-GtkType
+GType
 bg_ext_handler_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 
 	if (!type)
 	{
-		GtkTypeInfo info =
+		GTypeInfo info =
 		{
-			"BgExtHandler",
-			sizeof (BgExtHandler),
 			sizeof (BgExtHandlerClass),
-			(GtkClassInitFunc) bg_ext_handler_class_init,
-			(GtkObjectInitFunc) bg_ext_handler_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL
+			NULL,
+			NULL,
+			(GClassInitFunc) bg_ext_handler_class_init,
+			NULL,
+			NULL,
+			sizeof (BgExtHandler),
+			0,	
+			(GInstanceInitFunc) bg_ext_handler_init,
+			NULL
 		};
 
-		type = gtk_type_unique (mtm_ext_handler_get_type (), &info);
-	};
+		type = g_type_register_static (MTM_EXT_HANDLER_TYPE,
+					       "BgExtHandler",
+					       &info, 0);
+	}
 
 	return type;
 }
@@ -207,13 +259,13 @@ bg_ext_handler_new (MtmEnv *env)
 
 	g_return_val_if_fail (env != NULL, NULL);
 
-	handler = gtk_type_new (bg_ext_handler_get_type ());
+	handler = g_object_new (bg_ext_handler_get_type (), NULL);
 	MTM_STATEFUL (handler)->env = env;
 	return handler;
 }
 
-static void bg_ext_handler_set_arg (GtkObject *object, GtkArg *arg,
-	guint arg_id)
+static void
+bg_ext_handler_set_arg (GObject *object, guint arg_id, const GValue *value, GParamSpec *spec)
 {
 	BgExtHandler *handler = BG_EXT_HANDLER (object);
 	BGPreferences *config;
@@ -224,50 +276,50 @@ static void bg_ext_handler_set_arg (GtkObject *object, GtkArg *arg,
 		case ARG_COLOR1:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			color = GTK_VALUE_POINTER (*arg);
+			color = g_value_get_boxed (value);
 			*(config->color1) = *color;
 			break;
 		case ARG_COLOR2:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			color = GTK_VALUE_BOXED (*arg);
+			color = g_value_get_boxed (value);
 			*(config->color2) = *color;
 			break;
 		case ARG_COLOR_GRADIENT:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			config->gradient_enabled = GTK_VALUE_BOOL (*arg);
+			config->gradient_enabled = g_value_get_boolean (value);
 			break;
 		case ARG_GRADIENT_TYPE:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			config->orientation = GTK_VALUE_ENUM (*arg);
+			config->orientation = g_value_get_enum (value);
 			break;
 		case ARG_WALLPAPER_LAYOUT:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			config->wallpaper_type = GTK_VALUE_ENUM (*arg);
+			config->wallpaper_type = g_value_get_enum (value);
 			break;
 		case ARG_EXT_CONTEXT:
-			handler->priv->ext = MTM_EXT (GTK_VALUE_OBJECT (*arg));
-			config = gtk_object_get_data (
-				GTK_OBJECT (handler->priv->ext), "bgconfig");
+			handler->priv->ext = MTM_EXT (g_value_get_object (value));
+			config = g_object_get_data (
+				G_OBJECT (handler->priv->ext), "bgconfig");
 			if (!config)
 			{
 				config = BG_PREFERENCES (bg_preferences_new ());
 				bg_preferences_load (config);
 				handler->priv->configs = g_list_append (
 					handler->priv->configs, config);
-				gtk_object_set_data (
-					GTK_OBJECT (handler->priv->ext),
+				g_object_set_data (
+					G_OBJECT (handler->priv->ext),
 					"bgconfig", config);
 			}
 			break;
 	}
 }
 
-static void bg_ext_handler_get_arg (GtkObject *object, GtkArg *arg,
-	guint arg_id)
+static void
+bg_ext_handler_get_arg (GObject *object, guint arg_id, GValue *value, GParamSpec *spec)
 {
 	BgExtHandler *handler = BG_EXT_HANDLER (object);
 	BGPreferences *config;
@@ -277,31 +329,30 @@ static void bg_ext_handler_get_arg (GtkObject *object, GtkArg *arg,
 		case ARG_COLOR1:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			GTK_VALUE_BOXED (*arg) = config->color1;
+			g_value_set_boxed (value, config->color1);
 			break;
 		case ARG_COLOR2:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			GTK_VALUE_BOXED (*arg) = config->color2;
+			g_value_set_boxed (value, config->color2);
 			break;
 		case ARG_COLOR_GRADIENT:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			GTK_VALUE_BOOL (*arg) = config->gradient_enabled;
+			g_value_set_boolean (value, config->gradient_enabled);
 			break;
 		case ARG_GRADIENT_TYPE:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			GTK_VALUE_ENUM (*arg) = config->orientation;
+			g_value_set_enum (value, config->orientation);
 			break;
 		case ARG_WALLPAPER_LAYOUT:
 			config = bg_ext_handler_get_config (handler);
 			g_return_if_fail (config != NULL);
-			GTK_VALUE_ENUM (*arg) = config->wallpaper_type;
+			g_value_set_enum (value, config->wallpaper_type);
 			break;
 		case ARG_EXT_CONTEXT:
-			GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (
-				handler->priv->ext);
+			g_value_set_object (value, G_OBJECT (handler->priv->ext));
 			break;
 	}
 }
@@ -309,10 +360,18 @@ static void bg_ext_handler_get_arg (GtkObject *object, GtkArg *arg,
 BGPreferences*
 bg_ext_handler_get_config (BgExtHandler *handler)
 {
+	BGPreferences *prefs;
 	g_return_val_if_fail (handler != NULL, NULL);
 	g_return_val_if_fail (handler->priv->ext != NULL, NULL);
-	return gtk_object_get_data (
-		GTK_OBJECT (handler->priv->ext), "bgconfig");	
+	prefs = g_object_get_data (G_OBJECT (handler->priv->ext), "bgconfig");	
+	
+	if (strcmp (prefs->wallpaper_filename, handler->priv->ext->file))
+	{
+		g_free (prefs->wallpaper_filename);
+		prefs->wallpaper_filename = g_strdup (handler->priv->ext->file);
+	}
+
+	return prefs;
 }
 
 static gchar*
